import * as i0 from '@angular/core';
import { Component, ViewEncapsulation, ViewChild, Input, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

var Separator;
(function (Separator) {
    Separator[Separator["NA"] = 0] = "NA";
    Separator[Separator["Big"] = 1] = "Big";
    Separator[Separator["Small"] = 2] = "Small";
})(Separator || (Separator = {}));

const DefaultConfig = {
    WIDTH: 200,
    ARC_STROKE: 5,
    ARROW_Y: 22.5,
    ARROW_WIDTH: 4,
    ARROW_PIN_RAD: 8,
    SL_NORM: 3,
    SL_MID_SEP: 7,
    SL_SEP: 10,
    SL_WIDTH: 2,
    TXT_MARGIN: 10,
    LIGHT_Y: 55,
    LIGHT_RADIUS: 10,
    S_FAC_Y: 80,
    DIGITAL_Y: 145,
    UNIT_Y: 155,
    MAX_PURE_SCALE_VAL: 1000,
    INIT_LINE_FREQ: 2,
    DEF_START: 225,
    DEF_END: 135,
};

const error = (text, throwErr) => {
    const msg = `GaugeComponent: ${text}`;
    if (throwErr) {
        throw new Error(msg);
    }
    console.error(msg);
};
const validate = (props) => {
    if (!props.max) {
        error('Missing "max" input property (or zero)', true);
    }
    if (props.max < 0) {
        error('"max" input property cannot be negative.', true);
    }
    if (!(0 <= props.arcStart && props.arcStart <= 359) ||
        !(0 <= props.arcEnd && props.arcEnd <= 359)) {
        error('The scale arc end and start must be between 0 and 359 degrees.', true);
    }
    if (props.activateRedLightAfter && props.activateRedLightAfter > props.max) {
        error('The red light trigger value cannot be greater than the max value of the gauge.');
    }
    // if (props.scaleFactor && props.scaleFactor >= props.max) {
    //   showError('The factor cannot be greater than or equal to the max value.');
    // }
    if (props.sectors) {
        props.sectors.forEach((s) => {
            if (s.from < 0 || s.to < 0) {
                error('The sector bounds cannot be negative.', true);
            }
            if (s.from > props.max || s.to > props.max) {
                error('The sector bounds cannot be greater than the max value.', true);
            }
            if (s.from >= s.to) {
                error('The lower bound of the sector cannot be greater than or equal to the upper one.', true);
            }
        });
    }
};

function copySectors(sectors) {
    return sectors.map((s) => ({ ...s }));
}
class GaugeComponent {
    constructor(_renderer) {
        this._renderer = _renderer;
        this.viewBox = '';
        this.scaleLines = [];
        this.scaleValues = [];
        this.sectorArcs = [];
        this.radius = 0;
        this.center = 0;
        this.scaleFactor = 0;
        this._arcEnd = 0;
        this._value = 0;
        this._max = 0;
        this._mappedSectors = [];
    }
    /**
     * The current value of the gauge
     */
    set value(val) {
        this._value = Math.min(val, this._max);
        this._updateArrowPos(this._value);
    }
    get value() {
        return this._value;
    }
    /**
     * The maximal value of the gauge. It is suggested to use a number that is divisible by 10^n (e.g. 100, 1000, etc.)
     */
    // Note(Georgi): Don't use { require: true } since it's v16+ only
    set max(val) {
        if (this._max) {
            this._max = val;
            validate(this);
            this._initialize();
        }
        this._max = val;
    }
    get max() {
        return this._max;
    }
    get arc() {
        return this._arc(0, this._arcEnd);
    }
    get gaugeRotationAngle() {
        return this._arcEnd - this.arcEnd;
    }
    ngOnInit() {
        this.config = { ...DefaultConfig, ...this.config };
        if (!this.arcStart) {
            this.arcStart = this.config.DEF_START;
        }
        if (!this.arcEnd) {
            this.arcEnd = this.config.DEF_END;
        }
        validate(this);
        const width = this.config.WIDTH + this.config.ARC_STROKE;
        this.viewBox = `0 0 ${width} ${width}`;
        this.radius = this.config.WIDTH / 2;
        this.center = width / 2;
        this._arcEnd = this.arcEnd;
        if (this.arcStart > this.arcEnd) {
            this._arcEnd += 360 - this.arcStart;
        }
        else {
            this._arcEnd -= this.arcStart;
        }
        this._initialize();
    }
    ngAfterViewInit() {
        this._rotateGauge();
    }
    /**
     * Initialize gauge.
     */
    _initialize() {
        this.scaleLines = [];
        this.scaleValues = [];
        this._calculateSectors();
        this._updateArrowPos(this._value);
        this.scaleFactor = this._determineScaleFactor();
        this._createScale();
    }
    /**
     * Calculate arc.
     */
    _arc(start, end) {
        const largeArc = end - start <= 180 ? 0 : 1;
        const startCoor = this._getAngleCoor(start);
        const endCoor = this._getAngleCoor(end);
        return `M ${endCoor.x} ${endCoor.y} A ${this.radius} ${this.radius} 0 ${largeArc} 0 ${startCoor.x} ${startCoor.y}`;
    }
    /**
     * Get angle coordinates (Cartesian coordinates).
     */
    _getAngleCoor(degrees) {
        const rads = ((degrees - 90) * Math.PI) / 180;
        return {
            x: this.radius * Math.cos(rads) + this.center,
            y: this.radius * Math.sin(rads) + this.center,
        };
    }
    /**
     * Calculate/translate the user-defined sectors to arcs.
     */
    _calculateSectors() {
        if (!this.sectors) {
            return;
        }
        this._mappedSectors = copySectors(this.sectors);
        this._mappedSectors.forEach((s) => {
            const ratio = this._arcEnd / this.max;
            s.from *= ratio;
            s.to *= ratio;
        });
        this.sectorArcs = this._mappedSectors.map((s) => ({
            path: this._arc(s.from, s.to),
            color: s.color,
        }));
    }
    /**
     * Update the position of the arrow based on the current value.
     */
    _updateArrowPos(value) {
        if (!this.arrow) {
            return;
        }
        const pos = (this._arcEnd / this.max) * value;
        this._renderer.setStyle(this.arrow.nativeElement, 'transform', `rotate(${pos}deg)`);
    }
    /**
     * Rotate the gauge based on the start property. The CSS rotation, saves additional calculations with SVG.
     */
    _rotateGauge() {
        if (!this.gauge) {
            return;
        }
        const angle = 360 - this.arcStart;
        this._renderer.setStyle(this.gauge.nativeElement, 'transform', `rotate(-${angle}deg)`);
    }
    /**
     * Determine the scale factor (10^n number; i.e. if max = 9000 then scale_factor = 1000)
     */
    _determineScaleFactor(factor = 10) {
        // Keep smaller factor until 3X
        if (this.max / factor > 30) {
            return this._determineScaleFactor(factor * 10);
        }
        return factor;
    }
    /**
     * Determine the line frequency which represents after what angle we should put a line.
     */
    _determineLineFrequency() {
        const separators = this.max / this.scaleFactor;
        const separateAtAngle = this._arcEnd / separators;
        let lineFrequency;
        // If separateAtAngle is not an integer, use its value as the line frequency.
        if (separateAtAngle % 1 !== 0) {
            lineFrequency = separateAtAngle;
        }
        else {
            lineFrequency = this.config.INIT_LINE_FREQ * 2;
            for (lineFrequency; lineFrequency <= separateAtAngle; lineFrequency++) {
                if (separateAtAngle % lineFrequency === 0) {
                    break;
                }
            }
        }
        return lineFrequency;
    }
    /**
     * Checks whether the line (based on index) is big or small separator.
     */
    _isSeparatorReached(idx, lineFrequency) {
        const separators = this.max / this.scaleFactor;
        const totalSeparators = this._arcEnd / lineFrequency;
        const separateAtIdx = totalSeparators / separators;
        if (idx % separateAtIdx === 0) {
            return Separator.Big;
        }
        else if (idx % (separateAtIdx / 2) === 0) {
            return Separator.Small;
        }
        return Separator.NA;
    }
    /**
     * Creates the scale.
     */
    _createScale() {
        const accumWith = this._determineLineFrequency() / 2;
        const isAboveSuitableFactor = this.max / this.scaleFactor > 10;
        let placedVals = 0;
        for (let alpha = 0, i = 0; alpha >= -1 * this._arcEnd; alpha -= accumWith, i++) {
            let lineHeight = this.config.SL_NORM;
            const sepReached = this._isSeparatorReached(i, accumWith);
            // Set the line height based on its type
            switch (sepReached) {
                case Separator.Big:
                    placedVals++;
                    lineHeight = this.config.SL_SEP;
                    break;
                case Separator.Small:
                    lineHeight = this.config.SL_MID_SEP;
                    break;
            }
            // Draw the line
            const higherEnd = this.center - this.config.ARC_STROKE - 2;
            const lowerEnd = higherEnd - lineHeight;
            const alphaRad = (Math.PI / 180) * (alpha + 180);
            const sin = Math.sin(alphaRad);
            const cos = Math.cos(alphaRad);
            const color = this._getScaleLineColor(alpha);
            this._addScaleLine(sin, cos, higherEnd, lowerEnd, color);
            // Put a scale value
            if (sepReached === Separator.Big) {
                const isValuePosEven = placedVals % 2 === 0;
                const isLast = alpha <= -1 * this._arcEnd;
                if (!(isAboveSuitableFactor && isValuePosEven && !isLast)) {
                    this._addScaleValue(sin, cos, lowerEnd, alpha);
                }
            }
        }
    }
    /**
     * Get the scale line color from the user-provided sectors definitions.
     */
    _getScaleLineColor(alpha) {
        alpha *= -1;
        let color = '';
        if (this._mappedSectors.length) {
            this._mappedSectors.forEach((s) => {
                if (s.from <= alpha && alpha <= s.to) {
                    color = s.color;
                }
            });
        }
        return color;
    }
    /**
     * Add a scale line to the list that will be later rendered.
     */
    _addScaleLine(sin, cos, higherEnd, lowerEnd, color) {
        this.scaleLines.push({
            from: {
                x: sin * higherEnd + this.center,
                y: cos * higherEnd + this.center,
            },
            to: {
                x: sin * lowerEnd + this.center,
                y: cos * lowerEnd + this.center,
            },
            color,
        });
    }
    /**
     * Add a scale value.
     */
    _addScaleValue(sin, cos, lowerEnd, alpha) {
        let val = Math.round(alpha * (this.max / this._arcEnd)) * -1;
        let posMargin = this.config.TXT_MARGIN * 2;
        // Use the multiplier instead of the real value, if above MAX_PURE_SCALE_VAL (i.e. 1000)
        if (this.max > this.config.MAX_PURE_SCALE_VAL) {
            val /= this.scaleFactor;
            val = Math.round(val * 100) / 100;
            posMargin /= 2;
        }
        this.scaleValues.push({
            text: val.toString(),
            coor: {
                x: sin * (lowerEnd - posMargin) + this.center,
                y: cos * (lowerEnd - posMargin) + this.center,
            },
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: GaugeComponent, deps: [{ token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.1.2", type: GaugeComponent, selector: "ng2-gauge", inputs: { size: "size", arcStart: "arcStart", arcEnd: "arcEnd", sectors: "sectors", unit: "unit", digitalDisplay: "digitalDisplay", activateRedLightAfter: "activateRedLightAfter", darkTheme: "darkTheme", config: "config", value: "value", max: "max" }, viewQueries: [{ propertyName: "gauge", first: true, predicate: ["gauge"], descendants: true }, { propertyName: "arrow", first: true, predicate: ["arrow"], descendants: true }], ngImport: i0, template: "<section\n  class=\"ng2-gauge\"\n  [class.light]=\"darkTheme\"\n  [style.width]=\"size + 'px'\"\n>\n  <svg class=\"info\" [attr.viewBox]=\"viewBox\" xmlns=\"http://www.w3.org/2000/svg\">\n    <circle\n      *ngIf=\"activateRedLightAfter\"\n      class=\"red-light\"\n      [class.on]=\"value >= activateRedLightAfter\"\n      [attr.cx]=\"center\"\n      [attr.cy]=\"config.LIGHT_Y\"\n      [attr.r]=\"config.LIGHT_RADIUS\"\n    ></circle>\n    <text\n      *ngIf=\"max > config.MAX_PURE_SCALE_VAL\"\n      class=\"factor\"\n      [attr.x]=\"center\"\n      [attr.y]=\"config.S_FAC_Y\"\n    >\n      x{{ scaleFactor }} {{ unit }}\n    </text>\n    <text\n      *ngIf=\"digitalDisplay\"\n      class=\"digital\"\n      [attr.x]=\"center\"\n      [attr.y]=\"config.DIGITAL_Y\"\n    >\n      {{ value }}\n    </text>\n    <text class=\"unit\" [attr.x]=\"center\" [attr.y]=\"config.UNIT_Y\">\n      {{ unit }}\n    </text>\n  </svg>\n  <svg #gauge [attr.viewBox]=\"viewBox\" xmlns=\"http://www.w3.org/2000/svg\">\n    <path\n      class=\"main-arc\"\n      [attr.d]=\"arc\"\n      [attr.stroke-width]=\"config.ARC_STROKE\"\n      fill=\"none\"\n    />\n    <path\n      *ngFor=\"let arc of sectorArcs\"\n      [attr.d]=\"arc.path\"\n      [attr.stroke]=\"arc.color\"\n      [attr.stroke-width]=\"config.ARC_STROKE\"\n      fill=\"none\"\n    />\n    <line\n      *ngFor=\"let line of scaleLines\"\n      [attr.stroke-width]=\"config.SL_WIDTH\"\n      [attr.stroke]=\"line.color || (!darkTheme ? '#333' : '#fff')\"\n      [attr.x1]=\"line.from.x\"\n      [attr.y1]=\"line.from.y\"\n      [attr.x2]=\"line.to.x\"\n      [attr.y2]=\"line.to.y\"\n    />\n    <text\n      *ngFor=\"let val of scaleValues\"\n      class=\"text-val\"\n      dominant-baseline=\"central\"\n      [attr.x]=\"val.coor.x\"\n      [attr.y]=\"val.coor.y\"\n      [attr.transform]=\"\n        'rotate(' +\n        gaugeRotationAngle +\n        ', ' +\n        val.coor.x +\n        ', ' +\n        val.coor.y +\n        ')'\n      \"\n    >\n      {{ val.text }}\n    </text>\n    <rect\n      #arrow\n      class=\"arrow\"\n      [attr.x]=\"center - config.ARROW_WIDTH / 2\"\n      [attr.y]=\"config.ARROW_Y\"\n      [attr.height]=\"center - config.ARROW_Y\"\n      [attr.width]=\"config.ARROW_WIDTH\"\n      [attr.rx]=\"config.ARROW_WIDTH / 2\"\n      [attr.ry]=\"config.ARROW_WIDTH / 2\"\n    ></rect>\n    <circle\n      class=\"arrow-pin\"\n      [attr.cx]=\"center\"\n      [attr.cy]=\"center\"\n      [attr.r]=\"config.ARROW_PIN_RAD\"\n    />\n  </svg>\n</section>\n", styles: ["@font-face{font-family:Orbitron;font-style:normal;font-weight:700;src:local(\"Orbitron Bold\"),local(\"Orbitron-Bold\"),url(https://fonts.gstatic.com/s/orbitron/v8/Y82YH_MJJWnsH2yUA5AuYY4P5ICox8Kq3LLUNMylGO4.woff2) format(\"woff2\");unicode-range:U+0000-00FF,U+0131,U+0152-0153,U+02C6,U+02DA,U+02DC,U+2000-206F,U+2074,U+20AC,U+2212,U+2215}.ng2-gauge{position:relative;width:400px}.ng2-gauge .info{position:absolute;top:0;left:0}.ng2-gauge .arrow{transform-origin:50% 50%;fill:orange}.ng2-gauge text{font-family:Orbitron,sans-serif;font-weight:700;text-anchor:middle;fill:#333}.ng2-gauge.light text{fill:#fff}.ng2-gauge .text-val{font-size:12px}.ng2-gauge .arrow-pin{fill:#333}.ng2-gauge .main-arc{stroke:#333}.ng2-gauge.light .main-arc{stroke:#fff}.ng2-gauge .factor{font-size:7px}.ng2-gauge .digital{font-size:16px}.ng2-gauge .unit{font-size:10px}.ng2-gauge .red-light{fill:#ff4f4f;opacity:.1}.ng2-gauge .red-light.on{opacity:1}\n"], dependencies: [{ kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: GaugeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ng2-gauge', encapsulation: ViewEncapsulation.None, template: "<section\n  class=\"ng2-gauge\"\n  [class.light]=\"darkTheme\"\n  [style.width]=\"size + 'px'\"\n>\n  <svg class=\"info\" [attr.viewBox]=\"viewBox\" xmlns=\"http://www.w3.org/2000/svg\">\n    <circle\n      *ngIf=\"activateRedLightAfter\"\n      class=\"red-light\"\n      [class.on]=\"value >= activateRedLightAfter\"\n      [attr.cx]=\"center\"\n      [attr.cy]=\"config.LIGHT_Y\"\n      [attr.r]=\"config.LIGHT_RADIUS\"\n    ></circle>\n    <text\n      *ngIf=\"max > config.MAX_PURE_SCALE_VAL\"\n      class=\"factor\"\n      [attr.x]=\"center\"\n      [attr.y]=\"config.S_FAC_Y\"\n    >\n      x{{ scaleFactor }} {{ unit }}\n    </text>\n    <text\n      *ngIf=\"digitalDisplay\"\n      class=\"digital\"\n      [attr.x]=\"center\"\n      [attr.y]=\"config.DIGITAL_Y\"\n    >\n      {{ value }}\n    </text>\n    <text class=\"unit\" [attr.x]=\"center\" [attr.y]=\"config.UNIT_Y\">\n      {{ unit }}\n    </text>\n  </svg>\n  <svg #gauge [attr.viewBox]=\"viewBox\" xmlns=\"http://www.w3.org/2000/svg\">\n    <path\n      class=\"main-arc\"\n      [attr.d]=\"arc\"\n      [attr.stroke-width]=\"config.ARC_STROKE\"\n      fill=\"none\"\n    />\n    <path\n      *ngFor=\"let arc of sectorArcs\"\n      [attr.d]=\"arc.path\"\n      [attr.stroke]=\"arc.color\"\n      [attr.stroke-width]=\"config.ARC_STROKE\"\n      fill=\"none\"\n    />\n    <line\n      *ngFor=\"let line of scaleLines\"\n      [attr.stroke-width]=\"config.SL_WIDTH\"\n      [attr.stroke]=\"line.color || (!darkTheme ? '#333' : '#fff')\"\n      [attr.x1]=\"line.from.x\"\n      [attr.y1]=\"line.from.y\"\n      [attr.x2]=\"line.to.x\"\n      [attr.y2]=\"line.to.y\"\n    />\n    <text\n      *ngFor=\"let val of scaleValues\"\n      class=\"text-val\"\n      dominant-baseline=\"central\"\n      [attr.x]=\"val.coor.x\"\n      [attr.y]=\"val.coor.y\"\n      [attr.transform]=\"\n        'rotate(' +\n        gaugeRotationAngle +\n        ', ' +\n        val.coor.x +\n        ', ' +\n        val.coor.y +\n        ')'\n      \"\n    >\n      {{ val.text }}\n    </text>\n    <rect\n      #arrow\n      class=\"arrow\"\n      [attr.x]=\"center - config.ARROW_WIDTH / 2\"\n      [attr.y]=\"config.ARROW_Y\"\n      [attr.height]=\"center - config.ARROW_Y\"\n      [attr.width]=\"config.ARROW_WIDTH\"\n      [attr.rx]=\"config.ARROW_WIDTH / 2\"\n      [attr.ry]=\"config.ARROW_WIDTH / 2\"\n    ></rect>\n    <circle\n      class=\"arrow-pin\"\n      [attr.cx]=\"center\"\n      [attr.cy]=\"center\"\n      [attr.r]=\"config.ARROW_PIN_RAD\"\n    />\n  </svg>\n</section>\n", styles: ["@font-face{font-family:Orbitron;font-style:normal;font-weight:700;src:local(\"Orbitron Bold\"),local(\"Orbitron-Bold\"),url(https://fonts.gstatic.com/s/orbitron/v8/Y82YH_MJJWnsH2yUA5AuYY4P5ICox8Kq3LLUNMylGO4.woff2) format(\"woff2\");unicode-range:U+0000-00FF,U+0131,U+0152-0153,U+02C6,U+02DA,U+02DC,U+2000-206F,U+2074,U+20AC,U+2212,U+2215}.ng2-gauge{position:relative;width:400px}.ng2-gauge .info{position:absolute;top:0;left:0}.ng2-gauge .arrow{transform-origin:50% 50%;fill:orange}.ng2-gauge text{font-family:Orbitron,sans-serif;font-weight:700;text-anchor:middle;fill:#333}.ng2-gauge.light text{fill:#fff}.ng2-gauge .text-val{font-size:12px}.ng2-gauge .arrow-pin{fill:#333}.ng2-gauge .main-arc{stroke:#333}.ng2-gauge.light .main-arc{stroke:#fff}.ng2-gauge .factor{font-size:7px}.ng2-gauge .digital{font-size:16px}.ng2-gauge .unit{font-size:10px}.ng2-gauge .red-light{fill:#ff4f4f;opacity:.1}.ng2-gauge .red-light.on{opacity:1}\n"] }]
        }], ctorParameters: () => [{ type: i0.Renderer2 }], propDecorators: { gauge: [{
                type: ViewChild,
                args: ['gauge']
            }], arrow: [{
                type: ViewChild,
                args: ['arrow']
            }], size: [{
                type: Input
            }], arcStart: [{
                type: Input
            }], arcEnd: [{
                type: Input
            }], sectors: [{
                type: Input
            }], unit: [{
                type: Input
            }], digitalDisplay: [{
                type: Input
            }], activateRedLightAfter: [{
                type: Input
            }], darkTheme: [{
                type: Input
            }], config: [{
                type: Input
            }], value: [{
                type: Input
            }], max: [{
                type: Input
            }] } });

class GaugeModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: GaugeModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.1.2", ngImport: i0, type: GaugeModule, declarations: [GaugeComponent], imports: [CommonModule], exports: [GaugeComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: GaugeModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: GaugeModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [GaugeComponent],
                    imports: [CommonModule],
                    exports: [GaugeComponent],
                }]
        }] });

/*
 * Public API Surface of ng2-gauge
 */

/**
 * Generated bundle index. Do not edit.
 */

export { GaugeComponent, GaugeModule };
//# sourceMappingURL=ng2-gauge.mjs.map
